%% Stabilisierung der Bewegung der Grenzen des Beobachtungsfreiraums

function [imdata] = borderstabilization(imdata)

% Linke Grenze des Beobachtungsfreiraums abgreifen
links = arrayfun(@(x) x.borders(1), imdata);
% Rechte Grenze des Beobachtungsfreiraums abgreifen
rechts = arrayfun(@(x) x.borders(end), imdata);

% Breite des erkannten Beobachtungsfreiraum aus den Grenzen ermitteln
breitebeobachtungsfreiraum = max(abs(links-rechts));


%% Bereiche von Grenzen abschneiden, bei denen Grenzen auf Bildrand liegen
% Werte in linker Grenze abschneiden die auf linkem Bildrand liegen
links(links==1) = [];
% Werte in rechter Grenze abschneiden die auf rechtem Bildrand liegen
rechts(rechts==length(imdata(1).meancolumns)) = [];

% x-Koordinaten fr Fits bestimmen
xlinks = 1:length(links);
xrechts = 1:length(rechts);
% Linearer Fit an beide Grenzen
fitlinks = polyfit(xlinks, links, 1);
fitrechts = polyfit(xrechts, rechts, 1);

% Offset fr Verschiebung der Grenzen des Beobachtungsfreiraums nach innen
% in einer Anzahl von Pixeln
offset = 50;


%% Linke Grenze
% Gefittete Position der Grenze ber alle Frames eines Datensatzes
% berechnen
meanfitlinks = [mean([fitlinks(1) fitrechts(1)]) size(imdata(1).imginv, 2) + offset + 1];
meanfitvallinks = polyval(meanfitlinks, 1:length(imdata));
% Wenn eine Grenze rechts vom rechten Bildrand liegt, lege die Grenze auf
% den rechten Bildrand
meanfitvallinks(meanfitvallinks > size(imdata(1).imginv, 2)) = size(imdata(1).imginv, 2);
% Wenn eine Grenze links vom linken Bildrand liegt, lege die Grenze auf den
% linken Bildrand
meanfitvallinks(meanfitvallinks < 0) = 1;
% Position der Grenzen auf ganzzahligen Wert runden
linkskorrigiert = ceil(meanfitvallinks);


%% Rechte Grenze
% Gefittete Position der Grenze ber alle Frames eines Datensatzes
% berechnen
meanfitrechts = [mean([fitlinks(1) fitrechts(1)]) size(imdata(1).imginv, 2) + breitebeobachtungsfreiraum - offset + 1];
meanfitvalrechts = polyval(meanfitrechts, 1:length(imdata));
% Wenn eine Grenze rechts vom rechten Bildrand liegt, lege die Grenze auf
% den rechten Bildrand
meanfitvalrechts(meanfitvalrechts > size(imdata(1).imginv, 2)) = size(imdata(1).imginv, 2);
% Wenn eine Grenze links vom linken Bildrand liegt, lege die Grenze auf den
% linken Bildrand
meanfitvalrechts(meanfitvalrechts < 0) = 1;
% Position der Grenzen auf ganzzahligen Wert runden
rechtskorrigiert = ceil(meanfitvalrechts);

% Schleife um alle Frames, so dass Grenzen richtig gesetzt werden
for aa = 1:length(imdata)
    imdata(aa).borderskorrigiert = linkskorrigiert(aa):1:rechtskorrigiert(aa);
end

end