%% Optimierung der Transformationsmatrix
% Mit dieser Funktion wird die Transformationsmatrix optimiert, wobei als
% Optimierungsparameter der Korrelationskoeffizient R genutzt wird.
% Eingaenge in die Funktion sind:
%   - der aktuelle Frame J,
%   - der vorherige Frame I,
%   - die Position der ROI fuer den Crop
%   - das Element 3,1 der Transformationsmatrix
%   - die Laufvariable bb
% Bei der Optimierung wird nach der optimalen Transformationsmatrix
% gesucht, so dass das Testfeld des aktuellen Frames genau an die Position
% verschoben wird, wo auch das Testfeld des vorherigen Frames hinverschoben
% wurde.

function [T31opt] = correlationcoefficientoptimization(J, I, P, T31, bb)

% Funktion fuer die Optimierung definieren
fun = @(x) -corr2(imcrop(imwarp(J, affine2d([1 0 0; 0 1 0; x 0 1]), ...
    'OutputView', imref2d(size(J))), P), I);

% Optimierung des Matrixelements (3,1) fuer die Translation des Bildes, das
% Optimum liegt in einem Bereich zwischen der Verschiebung des vorherigen
% und des naechsten Frames.
T31opt = fminbnd(fun, T31(bb - 1), T31(bb + 1));

end