%% Funktion um ROI zu extrahieren
% Die ROI liegt hinter dem Druckspalt und hat eine Breite, die der halben
% Testfeldbreite entspricht.

function [imdata] = extractroi(imdata, n_DS)
%% Definitionen
% Massstab in mm/px
M = 0.0496;
% Kantenlaenge Testfeld in mm
K = 13;
% Kantenlaenge Testfeld in px
k = K/M;

%% Erstes relevantes Bild fuer die ROI ermitteln
% Das erste relevante Bild fuer die ROI ist das Bild, in dem die linke
% Kante des maskierten Testfelds auf der Bildspalte des Druckspalts liegt. 
% Es gilt n_DS = n_ML

% Bildspalte der linken Kante der Maskierung des Testfelds in einem 
% Bilddatensatz ermitteln
n_ML = arrayfun(@(x) find(any(x.testfield_mask),1),imdata);

% Abstand zwischen Bildspalte auf der die linke Kante der Maskierung des
% Testfelds und der Bildspalte auf der der Druckspalt liegt ermitteln.
delta_n_ML_DS = n_ML - n_DS(2);

% Index des Minimums/der Minima des Abstands ermitteln, um Bildindex i zu
% finden fuer den die linke Kante der Maskierung des Testfelds
% moeglichst nah an der Bildspalte des Druckspalts liegt und das Bild mit 
% Bildindex i_Start als Startbild definieren.
minidx_delta_n_ML_DS = find(abs(delta_n_ML_DS)==min(abs(delta_n_ML_DS)));
% Pruefen ob mehrere Minima gefunden wurden
if length(minidx_delta_n_ML_DS) > 1
    % Bei mehreren Minima wird der Index des ersten Minimums genutzt
    i_Start = minidx_delta_n_ML_DS(1);
else
    % Bei einem Minimum wird der Index des Minimums genutzt
    i_Start = minidx_delta_n_ML_DS;
end


%% Letzten relevanten Frame fuer die ROI ermitteln
% Definition der Bildspalte, die eine halbe Testfeldbreite vom Druckspalt
% nach links entfernt liegt
n_RL = round(n_DS(2) - k/2);

% Bildspalte der rechten Kante der Maskierung des Testfelds in einem 
% Bilddatensatz ermitteln
n_MR = arrayfun(@(x) find(any(x.testfield_mask),1,'last'),imdata);

% Bildindex i finden fuer den die rechte Kante der Maskierung des Testfelds
% auf der linken Kante der ROI liegt und als Endbild definieren
i_Ende = find(n_MR==n_RL);

% Abstand zwischen Bildspalte auf der die rechte Kante der Maskierung des
% Testfelds und der Bildspalte auf der die linke Kante der ROI liegt 
% ermitteln.
delta_n_MR_RL = n_MR - n_RL;

% Index des Minimums/der Minima des Abstands ermitteln, um Bildindex i zu
% finden fuer den die linke Kante der Maskierung des Testfelds
% moeglichst nah an der Bildspalte des Druckspalts liegt und das Bild mit 
% Bildindex i_Start als Startbild definieren.
minidx_delta_n_MR_RL = find(abs(delta_n_MR_RL)==min(abs(delta_n_MR_RL)));

% Pruefen ob mehrere Minima gefunden wurden
if length(minidx_delta_n_MR_RL) > 1
    % Bei mehreren Minima wird der Index des ersten Minimums genutzt
    i_Ende = minidx_delta_n_MR_RL(1);
else
    % Bei einem Minimum wird der Index des Minimums genutzt
    i_Ende = minidx_delta_n_MR_RL;
end


% Pruefen, ob der Index fuer das Endbild gefunden wurde. Wenn nicht, wird
% das letzte Bild des Bilddatensatzes als Endbild gesetzt.
if isempty(i_Ende)
    i_Ende = length(imdata);
end


%% Reduzierung auf relevante Frames
imdata = imdata(i_Start:i_Ende);


%% Crop der ROI
% Crop-Position der ROI definieren
CP = [n_RL 0 k/2 size(imdata(1).testfield_mask,2)];

% Schleife um alle Bilder eines Bilddatensatzes
for aa = 1:length(imdata)
    % Crop der ROI durchfuehren
    imdata(aa).roi = ...
        imcrop(imdata(aa).testfield_mask.*imdata(aa).imginv, CP);
    % Crop von der Maskierung des Testfelds durchfhren
    imdata(aa).roi_mask = imcrop(imdata(aa).testfield_mask, CP);
end % aa = 1:length(imdata)

end