%% Aufbau einer Filterung der Muster

function [imdata] = filterfingerpattern(imdata, f_R, alpha_R)

%% Berechnung Filtergroesse
% Massstab in mum/px
M = 49.6;

% Wellenlaenge Raster in mum
lambda_R = (f_R * 1e-4)^-1;

% Wellenlaenge Raster in px
lambda_Rpx = lambda_R/M;

% Wellenlaenge Raster vertikal in px
lambda_Rpxver = 2 * sin(alpha_R * (pi/180)) * lambda_Rpx;

% Filtergroesse bestimmen und in ungerade Zahl umwandeln
w = ceil(2 * lambda_Rpxver);
if mod(w,2) == 0
    w = w + 1;
end

% Filtergroesse um -2 und +2 variieren
filtersizes = [w-2, w, w+2];

%% Filterung durchfuehren
% Schleife um alle Bilder eines Bilddatensatzes
for aa = 1:length(imdata)
    % Laufvariable fuer die Reihe der gefilterten Bilder definieren. Alle
    % drei Gauss-Bilder werden mit jedem Cosinus-Filter kombiniert. Laeuft
    % von 1 bis 9.
    dd = 1;
    % Erste Schleife um alle Filtergroessen fuer Gauss-Filterung
    for bb = 2%1:length(filtersizes)
        % Gauss-Filterung basierend auf Rasterfrequenz durchfuehren
        imdata(aa).roigfilt(:,:,bb) = imgaussfilt(imdata(aa).roi, 2, ...
            'FilterSize', [filtersizes(bb) filtersizes(bb)]);
        
        % Gaussgefiltertes Bild mit Maske multiplizieren
        imdata(aa).roigfilt(:,:,bb) = imdata(aa).roigfilt(:,:,bb) .* ...
            imdata(aa).roi_mask;
        
        % Zweite Schleife um alle Filtergroessen fuer Cosinus-Filterung
        for cc = 2%1:length(filtersizes)
            % Standard-Cosinus-Filter fuer horizontale Strukturen 
            % definieren. Groesse basiert auf Rasterfrequenz
            CF = repmat(cos(linspace(-pi,pi,filtersizes(cc)))' ...
                ,1,filtersizes(cc));
            
            % Gaussgefiltertes Bild abgreifen
            I = imdata(aa).roigfilt(:,:,bb);
            
            % Alle Nullen auf NaN setzen
            I(I == 0) = NaN;
            
            % Filterung mit Filtermatrix CF, rescale und Quadration
            % durchfuehren
            J = rescale(imfilter(I, CF, 'symmetric'),0,1).^2;
            
            % Filter-Bild in imdata-Struct ablegen
            imdata(aa).roicosfiltsq(:,:,dd) = J;
            
            % Laufvariable um 1 vergroessern
            dd = dd + 1;

        end % for cc = 1:length(filterizes)
    end % for bb = 1:length(filtersizes)
    % Maximum-Superposition (wenn mehrere Filtergroessen kombiniert werden)
    imdata(aa).roifingerpattern = max(imdata(aa).roicosfiltsq,[],3).^2; 
end % for aa = 1:length(imdata)


end