%% Erkennung des Beobachtungsfreiraums
% Mit diesem Skript werden die relevanten Frames erkannt und die Frames
% geloescht, welche keine relevanten Bilddaten enthalten.

function [imdata] = findacquisitiongap(imdata)


%% Identifizierung der relevanten Frames
% Relevante Frames sind Frames, in denen der Beobachtungsfreiraum sichtbar
% ist. Dazu werden die Grenzen (borders) des Beobachtungsfreiraum zuerst
% identifiziert

% Schleife um alle Frames eines Datensatzes
for aa = 1:length(imdata)
    % Mittelwert jeder Spalte eines Frames berechnen
    imdata(aa).meancolumns = mean(imdata(aa).imginv,1);
    
    % Schwellwert der Intensitaet aus erstem Frame ermitteln, mit dem die
    % Front des Substrattraegers ermittelt wird
    threshold = 0.99 * mean(imdata(1).meancolumns);

    % Suche Punkte unterhalb des Schwellwerts. Alle Punkte mit Werten
    % unterhalb des Schwellwerts befinden sich im Beobachtungsfreiraum
    imdata(aa).borders = find(imdata(aa).meancolumns < threshold);
end


%% Entfernen der irrelevanten Frames
% Entfernen der Frames, in denen der Beobachtungsfreiraum nicht vorhanden
% ist
imdata(arrayfun(@(x) isempty(x.borders), imdata)) = [];


%% Korrektur der Bewegung
[imdata] = borderstabilization(imdata);

% Entfernen aller Frames in denen die Lnge des Beobachtungsfreiraums
% gleich Eins ist
imdata(arrayfun(@(x) length(x.borderskorrigiert)==1, imdata)) = [];


%% Maskierung der Bereiche ausserhalb des Beobachtungsfreiraums
for bb = 1:length(imdata)
    % Identifizieren aller Spalten die maskiert werden sollen. Das sind
    % alle Spalten die ausserhalb der Grenzen des Beobachtungsfreiraums
    % liegen
    maskidx = ~ismember(1:size(imdata(bb).imginv,2), imdata(bb).borderskorrigiert);
    
    % Bild fuer Maskierung abgreifen und als img_masked in imdata-Struct
    % ablegen
    imdata(bb).imgmasked = imdata(bb).imginv;
    
    % Maskierung des Bilds durch Setzen der Bildspalten auf NaN
    imdata(bb).imgmasked(:,maskidx) = NaN;
end

end