%% Funktion um Testfeld zu finden

function [imdata] = findtestfield(imdata, n_DS, InputList, nDataset)

% Maske erstellen, die eine Spalte mit Einsen beinhaltet, wo Druckspalt
% ist, ansonsten nur Nullen besitzt
nipmask = zeros(size(imdata(1).imgmasked, 1), size(imdata(1).imgmasked, 2));
nipmask(:,n_DS(2)) = 1;

% meancol-Vektor vordefinieren
meancol = zeros(1, length(imdata));
% Schleife ueber alle Frames
for aa = 1:length(imdata)
    % Mittelwert der Intensitaet an Druckspaltposition ueber ganze 
    % Bildspalte ermitteln
    meancol(aa) = mean(imdata(aa).imgmasked(:,n_DS(2)));
    
    % Schleife ausnutzen, um nipmask in imdata-struct zu schreiben
    imdata(aa).nipmask = nipmask;
end


%% Konstanten
% Geschwindigkeitaus Experiment ermitteln
v = calculatevelocity(InputList, nDataset);
% Massstab aus Video in mm/px
M = 0.0496;
% Kantenlaenge Testfeld in mm
K = 13;
% Kantenlaenge Testfeld in px
k = K/M;


%% Frames ermitteln in denen Testfeld mit einer Kante auf Druckspalt liegt
[i_n_TFL_DS, i_n_TFR_DS] = findchangeptsansatz(meancol, v, k);

% fig = figure('Name',InputList(nDataset).name, 'visible','off');
% plot(diff(meancol));
% % Plot als Tiff-Datei abspeichern
% saveas(fig,[pwd filesep '3_Output' ...
%     filesep InputList(nDataset).name(1:end-4) '.tif']);
% close all


%% Berechnung erstes relevantes Bild
% Position der linken Testfeldkante, wenn Testfeld erstmalig vollstaendig
% im Bild liegt
posleftedge_fullfield = size(imdata(1).imgmasked, 2) - k;

% Weg ermitteln, den Testfeld von erstmaligem vollstaendigem Vorhandensein
% bis zum Druckspalt zurueckgelegt hat
W = abs(n_DS(2) - posleftedge_fullfield);

% Anzahl an Frames ermitteln, die zwischen erstmaligem vollstaendigem
% Vorhandensein des Testfelds und zum Zeitpunkt der linken Kante an
% Druckspalt ermitteln
framesdiff = W/v;

% Ersten relevanten Frame berechnen
fstframe_fullfield = round(i_n_TFL_DS - framesdiff);

% imshow(imdata(fstframe_fullfield).imgmasked);


%% Berechnung letzter relevanter Frame V2
% Position der rechten Testfeldfeldkante, wenn Testfeld gerade noch links
% im Bild ist
posrightedge_fullfield = k;

% Weg ermitteln, den Testfeld von erstmaligem vollstaendigem Vorhandensein
% bis zum Druckspalt zurueckgelegt hat
W_lst = abs(n_DS(2) - posrightedge_fullfield);

% Anzahl an Frames ermitteln, die fr den Weg von Druckspalt bis zur
% Position der rechten Testfeldkante liegen, wenn Testfeld mit linker Kante
% auf Bildrahmen liegt.
framesdiff_nip2lstfull = W_lst/v;

% Letzter relevanter Frame
lstframe_fullfield = round(i_n_TFR_DS + framesdiff_nip2lstfull);

% figure;
% imshow(imdata(lstframe_fullfield).imgmasked);

% %% Speichern von Kontrollframes zur Pruefung
% % Maske mit Druckspaltposition erstellen. Dazu kommt noch die Position der
% % jeweils anderen Kante wenn eine der Testfeldkanten auf dem Druckspalt
% % liegt
% tfmask = zeros(size(imdata(1).imgmasked));
% tfmask(:,ceil(nipposition(2))) = 1;
% tfmask(:,ceil(nipposition(2) + k)) = 1;
% tfmask(:,ceil(nipposition(2) - k)) = 1;
% 
% % Bilder erstellen
% im_s = imfuse(tfmask, imdata(testfield_in).imgmasked);
% im_e = imfuse(tfmask, imdata(testfield_out).imgmasked);
% 
% % Bilder abspeichern
% imwrite(im_s, [pwd filesep '3_Output' filesep 'Kontrollframes' filesep InputList(nDataset).name(1:(end-4)) '_1.tif']);
% imwrite(im_e, [pwd filesep '3_Output' filesep 'Kontrollframes' filesep InputList(nDataset).name(1:(end-4)) '_2.tif']);


%% Reduzierung auf relevante Frames
imdata = imdata(fstframe_fullfield:lstframe_fullfield);


%% Bildstabilisierung
[imdata] = imagestabilization(imdata, v);


%% Finaler Crop auf Testfeld
% Beim finalen Crop werden rechts und links noch die Raender leicht
% abgeschnitten, so dass Randeffekte nicht betrachtet werden.
Pos_TF_final = [23.5 0.5 243 235];
for bb = 1:length(imdata)
    % Crop des Testfelds auf finale Groesse
    imdata(bb).testfield_stab = ...
        imcrop(imdata(bb).testfield_stab,Pos_TF_final);
    % Crop der Druckspalt-Maske auf finale ROI-Groesse
    imdata(bb).nipmask_stab = ...
        imcrop(imdata(bb).nipmask_stab,Pos_TF_final);
end %  bb = 1:length(imdata)


% %% Anpassung der mittleren Bildintensitaet auf 0.5
% [imdata] = equalizemeanimgintensity(imdata, 0.5);

end