%% Funktion zur Stabilisierung der Frames
% Mit dieser Funktion wird die Bewegung des Testfelds durch eine
% affine-Transformation und die imwarp-Funktion herausgerechnet. Durch 
% eine Optimierung der Transformationsmatrix in der Unter-Funktion
% correlationcoefficientoptimization werden Sub-Pixel Bewegungen und
% Spruenge entfernt.

function [imdata] = imagestabilization(imdata, v)
%% Definitionen
% Position der ROI bezogen auf maskierten Frame
Pos_TF = [490.5 0.5 278 235];

% Laufvariable fuer Optimierungsschleife ueber alle Frames
counter = round((size(imdata(1).imgmasked, 2) - Pos_TF(3) + 1) / v);

% Transformationsmatrix fuer Stabilisieurng auf Testfeld aufbauen. T_stab
% ist eine Matrix mit Einsen auf der Hauptdiagonale
T_stab = eye(3);

% Element 3,1 fuer Transformationsmatrix T_stab vordefinineren
T31=zeros(counter, 1);
% Werte T31 fuer Transformationsmatrix T_stab vorab ermitteln
for aa = 2:counter
    % Eintragung in Transformationsmatrix mit Geschwindigkeit vorbereiten
    T31(aa) = T31(aa - 1) + v;
end % yy = 2:(counter - 1)

% Transformationsmatrix fuer Testfeldmaskierung definieren
T_mask = eye(3);
% Testfeldmaskierung in erstem Bild definieren. Hier ist die Position der
% Maskierung ueber die Lage des Testfelds definiert.
imdata(1).testfield_mask = zeros(size(imdata(1).imgmasked));
% Einsen dahin wo Testfeld liegt
imdata(1).testfield_mask(:,515:515+243) = 1;

% Vektor fuer Korrelationskoeffizient zwischen aktuellem und naechstem
% Frame vordefinieren
R = zeros((counter - 1), 1);
% Beim ersten Frame liegt noch keine Verschiebung des Testfelds vor. Daher
% wuerde die ROI mit der ROI korreliert werden. Um Rechenzeit zu sparen,
% wird der Korrelationskoeffizient direkt auf 1 gesetzt.
R(1) = 1;

% Entfernen aller NaNs aus maskierten Frames, so dass imwarp funktioniert
for bb = 1:length(imdata)
    imdata(bb).imgpretrans = imdata(bb).imgmasked;
    imdata(bb).imgpretrans(isnan(imdata(bb).imgpretrans)) = 0;
end % bb = 1:length(imdata)

% Initialen transformierten Frame definieren
imdata(1).imgtrans = imdata(1).imgpretrans;
imdata(1).nipmasktrans = imdata(1).nipmask;

% Crop des Testfelds aus erstem Frame. Hier ist die Lage des Testfelds 
% bekannt.
imdata(1).testfield_stab = imcrop(imdata(1).imgtrans, Pos_TF);
imdata(1).nipmask_stab = imcrop(imdata(1).nipmasktrans, Pos_TF);

T31opt=zeros(counter, 1);


%% Schleife um alle Frames
% Schleife laeuft bis (counter - 1), da letzter Frame nicht mehr mit einem
% naechsten Frame korreliert werden kann.
for cc = 2:(counter - 1)
    % Optimierung des Elements 3,1 der Transformationsmatrix durchfuehren
    % und das Ergebnis in die Transformationsmatrix T schreiben
    T_stab(3, 1) = correlationcoefficientoptimization( ...
        imdata(cc).imgpretrans, imdata(cc - 1).testfield_stab, Pos_TF, T31, cc);
                
    % Tatsaechliche Transformation mit optimierter Transformationsmatrix 
    % durchfuehren
    imdata(cc).imgtrans = imwarp(imdata(cc).imgpretrans, affine2d(T_stab), ...
        'OutputView', imref2d(size(imdata(cc).imgpretrans)));
    % Crop der ROI aus dem transformierten Frame
    imdata(cc).testfield_stab = imcrop(imdata(cc).imgtrans, Pos_TF);
    
    % Korrelation zwischen transformiertem Frame und vorherigem Frame
    % berechnen
    R(cc) = corr2(imdata(cc).testfield_stab, imdata(cc - 1).testfield_stab);
    
    % Transformation zusaetzlich auf die Maske mit der Position des
    % Druckspalts durchfuehren
    imdata(cc).nipmasktrans = imwarp(imdata(cc).nipmask, affine2d(T_stab), ...
        'OutputView', imref2d(size(imdata(cc).nipmask)));
    % Crop der ROI aus dem transformierten Frame
    imdata(cc).nipmask_stab = imcrop(imdata(cc).nipmasktrans, Pos_TF);
    
    % Transformationsmatrix fuer Testfeldmaskierung aufbauen
    T_mask(3, 1) = -T_stab(3, 1);
    % Transformation der Testfeldmaskierung
    imdata(cc).testfield_mask = imwarp(imdata(1).testfield_mask, ...
        affine2d(T_mask),'OutputView', ...
        imref2d(size(imdata(1).testfield_mask)));
    
    
end % cc = 1:counter

% Reduzierung auf relevante Frames
imdata(arrayfun(@(x) isempty(x.imgtrans), imdata)) = [];

end