%% Hauptskript der Bilddatenverarbeitung
% Mit diesem Skript werden .mraw-Videos verarbeitet und die im Druckspalt
% auftretenden Musterbildungsphaenomene ausgewertet sowie charakterisiert.


%% Workspace loeschen, Offene Figures schliessen und Command Window leeren
clearvars;
close all;
clc


%% Import von Funktionen in die Funktionsbibliothek von Matlab
% Pruefen ob Ordner fuer Funktionen vorhanden ist
if ~isfolder([pwd filesep '0_Functions'])
    % Warnung ausgeben
    warning('Ordner mit relevanten Funktionen wurde nicht gefunden!')
    % Verarbeitung abbrechen
    return
else
    % Pfade zum Funktionen-Ordner und allen Unterordnern erstellen
    FunctionPath = genpath([pwd filesep '0_Functions' filesep]);
    % Pfade vom Funktionenordner der Funktionsbibliothek hinzufuegen und
    % bis zum Beenden von Matlab dort behalten
    addpath(FunctionPath,'-end'); 
end % ~isfolder([pwd filesep '9_MatlabFunctions'])


%% Input-Ordner definieren und Liste mit Videodatensaetzen erstellen
InputPath = [pwd filesep '1_Input'];
TempPath = [pwd filesep '2_Temp'];
OutputPath = [pwd filesep '3_Output'];
% Liste mit Dateien im Ordner erstellen, deren Dateityp .cih ist
InputList = dir([InputPath filesep 'Messdaten' filesep '*.cih']);
% Messdatensaetze auf die mit Fingerbildung reduzieren
[InputList] = reducedatasets(InputPath, InputList);
% Anzahl an Video-Datensaetzen im Inputordner
numofDatasets = length(InputList);


%% Ergebnis-Struct vordefinieren
fingerdata = struct;


%% Berechnung der Druckspaltposition aus den Referenzbildern
[nipposition] = calculatenipposition(InputPath);


%% Schleife um zu verarbeitende Messdatensaetze
% zu verarbeitende Messdatensaetze definieren
Datasets2process = 1:numofDatasets;
Datasetinprocess = 1:length(Datasets2process);
for nDataset = Datasets2process
    %% Anzeige Berechnungszeit 1/2
    % Timer starten
    tic
    % Status der Verarbeitung im Command Window anzeigen (aktueller
    % Datensatz in Bezug auf Anzahl aller zu verarbeitender Datensaetze)
    fprintf('Verarbeite Datensatz %d von %d... (%d/%d) ', nDataset, ...
        numofDatasets, Datasetinprocess(Datasets2process == nDataset), ...
        length(Datasets2process))
    
    
    %% Datensatz einlesen und vorbereiten
    [imdata] = readdataset(InputList, nDataset);
    
    
    %% Relevante Druckspaltposition aus nipposition-Struct abgreifen
    n_DS = getrelevantnipposition(InputList, nDataset, nipposition);

    
    %% Ermittlung relevanter Frames und Maskierung Beobachtungsfreiraum
    [imdata] = findacquisitiongap(imdata);
    
    
    %% Maskierung des Testfelds
    [imdata] = findtestfield(imdata, n_DS, InputList, nDataset);
    
    
    %% Extraktion der ROI hinter dem Druckspalt
    [imdata] = extractroi(imdata, n_DS);
    
    
    %% Filterung von Fingerstrukturen
    [imdata] = filterfingerpattern(imdata, ...
        str2double(InputList(nDataset).name(13:15)), 45);
    
    
    %% Auswertung von Fingeranzahl und Fingerfrequenz
    [imdata] = getfingernumberandfrequency(imdata);
    
    
    %% Speichern der Ergebnisse
%     [fingerdata] = saveresults(fingerdata, imdata, InputList, ...
%     nDataset, Datasets2process, TempPath, OutputPath);
    
    
    %% Anzeige Berechnungszeit 2/2
    % Timer beenden und Zeitmessung fuer Anzeige vorbereiten
    ElapsedTime = toc;
    % Print line im command window that processing of dataset ended
    fprintf('Fertig in %4.2f s.\n',ElapsedTime)


end % for aa = 1:DatasetNum