%% Funktion zum Reduzieren der Bilddatensaetze nach Sichtungstabelle

function [InputList] = reducedatasets(InputPath, InputList)

% Leeres Cell-Array zum Einlesen der Tabelle erstellen
experiments = cell(3, 185);

% Dateinamen der Excel-Tabelle definieren
tabname = '2020_03_17_Tabelle_Sichtung_Bilddatensaetze.xlsx';
% Pfad zur Tabelle definieren
tabpath = [InputPath filesep tabname];


%% Einlesen von Parametern der Druckversuche
% Nummer eines Druckversuchs aus der Tabelle einlesen
[exp_nr, ~] = xlsread(tabpath, 4, 'A:A');

% Tonwert eines Druckversuchs aus der Tabelle einlesen
[~, exp_T] = xlsread(tabpath, 4, 'C:C'); 
% Spaltenueberschrift aus Tabelle entfernen
exp_T(1) = [];

% Rasterfrequenz eines Druckversuchs aus der Tabelle einlesen
[~, exp_f_R] = xlsread(tabpath, 4, 'D:D'); 
% Spaltenueberschrift aus Tabelle entfernen
exp_f_R(1) = [];

% Geschwindigkeit eines Druckversuchs aus der Tabelle einlesen
[~, exp_v_P] = xlsread(tabpath, 4, 'E:E'); 
% Spaltenueberschrift aus Tabelle entfernen
exp_v_P(1) = [];

%% Einlesen der Parameter der Sichtung
% Fingerbildung in Druckversuch-Wiederholung 1
[FB1,~] = xlsread(tabpath, 4, 'I:I');
% Fingerbildung in Druckversuch-Wiederholung 2
[FB2,~] = xlsread(tabpath, 4, 'J:J');
% Fingerbildung in Druckversuch-Wiederholung 3
[FB3,~] = xlsread(tabpath, 4, 'K:K'); 
% Zusammensetzen eines Fingerbildungs-Vektor fuer die
% Druckversuch-Wiederholungen
FB = [FB1, FB2, FB3];



%% Dateinamen der Druckversuche mit Fingerbildung zusammensetzen
% Schleife ueber alle Druckversuche
for aa = 1:length(exp_nr)
    % Pruefen ob alle drei Wiederholungen Fingerbildung aufweisen
    if FB(aa,1)==1 && FB(aa,2)==1 && FB(aa,3)==1
        % Schleife ueber alle Wiederholungen
        for bb = 1:3
            experiments{bb, aa} = [num2str(exp_nr(aa),'%03d') '_' ...
                num2str(bb) '_' exp_T{aa} '_' exp_f_R{aa} '_' exp_v_P{aa}];
        end % bb = 1:3
    end
end % aa = 1:length(exp_nr)

% experiments Cell-Array von drei in eine Spalte umwandeln
experiments = reshape(experiments, [], 1);


%% InputList auf Bilddatensaetze mit Fingerbildung reduzieren
InputList = InputList(~cellfun(@isempty,experiments));

end